/*
 *
 * Copyrighted by NPAC, Syracuse Univ., 1994.
 *
 */

#include <stdio.h>

#ifndef DEM_H
#define DEM_H

/* All data a aligned and padded to 1024 byte block */

#define USGS_FILE_BLOCK_SIZE    1024
#define USGS_INT_FIELD_WIDTH       6
#define USGS_DOUBLE_FIELD_WIDTH   24
#define USGS_FLOAT_FIELD_WIDTH    12

#define TYPE_A_HEADER_NUM_CHAR
#define TYPE_A_HEADER_NUM_INT
#define TYPE_A_HEADER_NUM_DOBULE
#define TYPE_A_HEADER_NUM_FLOAT

#define TYPE_B_HEADER_NUM_INT      4
#define TYPE_B_HEADER_NUM_DOBULE   5

#define TYPE_C_HEADER_NUM_INT
#define TYPE_C_HEADER_NUM_DOBULE
#define TYPE_C_HEADER_NUM_FLOAT

struct DEM_header
{
   char   Name[145];
   int    Level_code;
   int    Pattern_code;
   int    Planimetric_code;
   int    Zone_code;
   double Map_Projection_Parameters[15];
   int    Ground_Planimetric_Coordinates_Units_Code;
   int    Elevation_Coordinates_Units_Code;
   int    Polygon_Sides;
   double Ground_Coordinates[4][2];
   double Max_Elevation;
   double Min_Elevation;
   double Rotation_Angle;
   int    Accuracy_Code;
   double Spatial_Resolution[3];

   int    Row_per_record;
   int    N;
   int    M;
};

typedef struct DEM_header DEM_header;

struct DEM_record
{
   int     m;
   int     n;
   int     M;
   int     Col_per_record;
   double  Coordinate[2];
   double  Base_Elevation;
   double  Max_Elevation;
   double  Min_Elevation;
   int    *Elevations;
};

typedef struct DEM_record DEM_record;

struct DEM_accuracy
{
   int RMSE_file_Available;
   int RMSE_file_x;
   int RMSE_file_y;
   int RMSE_file_z;
   int RMSE_file_Sample_size;

   int RMSE_DEM_Available;
   int RMSE_DEM_x;
   int RMSE_DEM_y;
   int RMSE_DEM_z;
   int RMSE_DEM_Sample_Size;
};

typedef struct DEM_accuracy DEM_accuracy;

/*
 *                    (M - 1, N - 1)
 *      ^             ur_x, ur_y
 *      |
 *      |
 *      |
 *      |
 *      |
 *(0, 0)|------------->
 * ll_x, ll_y
 */

struct DEM_matrix
{
   int   M;
   int   N;

   double Lower_Left_Corner[2];
   double Upper_Right_Corner[2];

   int **Elevations;
};

typedef struct DEM_matrix DEM_matrix;

struct DEM
{
   DEM_header   *Header;
   DEM_record  **records;
   DEM_matrix   *matrix;
   DEM_accuracy *Stat;
};

typedef struct DEM DEM;

#endif

DEM *read_DEM_file(char *DataFilename);
DEM_header *read_DEM_header(FILE *DataFile);
DEM_record *read_DEM_record(FILE *DataFile);

int read_int(FILE *DataFile, int field_width);
double read_double(FILE *DataFile, int field_width);

void print_DEM(DEM *DEM_data);
void print_DEM_header(DEM_header *Header);
void print_DEM_record(DEM_record *Record);


